@@ -1,26 +1,26 @@
-move_grid_grob <- function(loon_grob, index, swap, square_xy, temporary, ...) {
+move_grid_grob <- function(loon.grob, index, swap, squarexy, temporary, ...) {
   obj <- character(0)
-  class(obj) <- names(loon_grob$children)
+  class(obj) <- names(loon.grob$children)
   UseMethod("move_grid_grob", obj)
 }
 
 
 
-move_grid_grob.l_plot <- function(loon_grob, index, swap, square_xy, temporary, ...)  {
+move_grid_grob.l_plot <- function(loon.grob, index, swap, squarexy, temporary, ...)  {
   
-  move_jitter_grob.l_plot(loon_grob,
+  move_jitter_grob.l_plot(loon.grob,
                           index,
                           swap,
-                          jitter_xy  = square_xy,
+                          jitterxy  = squarexy,
                           temporary, ...)
 }
 
-move_grid_grob.l_graph <- function(loon_grob, index, swap, square_xy, temporary, ...)  {
+move_grid_grob.l_graph <- function(loon.grob, index, swap, squarexy, temporary, ...)  {
 
-  move_jitter_grob.l_graph(loon_grob,
+  move_jitter_grob.l_graph(loon.grob,
                            index,
                            swap,
-                           jitter_xy  = square_xy,
+                           jitterxy  = squarexy,
                            temporary, ...)
 }
 
@@ -36,8 +36,8 @@ square_coord <- function(x, y, index) {
   nm <- grDevices::n2mfrow(n)
   row_n <- nm[1]
   col_n <- nm[2]
-  step_x <- (max_x - min_x)/col_n
-  step_y <- (max_y - min_y)/row_n
+  stepX <- (max_x - min_x)/col_n
+  stepY <- (max_y - min_y)/row_n
   x <- c()
   y <- c()
 
@@ -46,8 +46,8 @@ square_coord <- function(x, y, index) {
       if((i-1)* col_n + j > n) {
         break
       } else {
-        x[(i-1)*col_n + j] <- min_x + step_x * (j - 1)
-        y[(i-1)*col_n + j] <- min_y + step_y * (i - 1)
+        x[(i-1)*col_n + j] <- min_x + stepX * (j - 1)
+        y[(i-1)*col_n + j] <- min_y + stepY * (i - 1)
       }
     }
   }
